import StyledLink from "@/components/gallery/StyledLink";
import { cn } from "@/lib/utils";
import { Fragment } from "react/jsx-runtime";


export function DescItem({ 
    name, 
    children, 
    className,
    icon 
}: {
    name: string;
    className?: string;
    children?: React.ReactNode;
    icon?: React.ReactNode;
}) {
    return <div className={cn("grid content-start", className)}>
        <div className="flex items-center gap-1.5 mb-1">
            {icon && <span className="text-muted-foreground">{icon}</span>}
            <span className="text-muted-foreground text-sm font-medium">{name}</span>
        </div>
        <div className="text-primary leading-relaxed font-medium pl-0.5">{children}</div>
    </div>;
}

export function DescTagItem({
    items, name, className, icon
}: {
    name: string;
    items: string[];
    className?: string;
    icon?: React.ReactNode;
}) {
    return <DescItem name={name} className={className} icon={icon}>
        {items.length === 0 ? (
            <span className="text-muted-foreground italic text-sm">정보 없음</span>
        ) : items.map(
            (x, i) => 
            <Fragment key={x}>
                <StyledLink 
                    to={`/search?allow_tag=${name.toLowerCase()}:${x}`}
                    className="inline-flex items-center hover:bg-secondary/40 px-1.5 py-0.5 rounded-md transition-colors">
                    {x}
                </StyledLink>
                {i + 1 < items.length && <span className="mx-0.5">, </span>}
            </Fragment>
        )}
    </DescItem>;
}