import { promises } from "node:fs";
import { join } from "node:path";
const { readdir } = promises;
import type { DocumentAccessor } from "../../model/doc.ts";
import type { IDiffWatcher } from "../watcher.ts";

function setupCommon(watcher: IDiffWatcher, basepath: string, initial_filenames: string[], cur: string[]) {
	// Todo : reduce O(nm) to O(n+m) using hash map.
	const added = cur.filter((x) => !initial_filenames.includes(x));
	const deleted = initial_filenames.filter((x) => !cur.includes(x));
	for (const it of added) {
		const cpath = join(basepath, it);
		watcher.emit("create", cpath);
	}
	for (const it of deleted) {
		const cpath = join(basepath, it);
		watcher.emit("delete", cpath);
	}
}
export async function setupHelp(watcher: IDiffWatcher, basepath: string, cntr: DocumentAccessor) {
	const initial_document = await cntr.findByPath(basepath);
	const initial_filenames = initial_document.map((x) => x.filename);
	const cur = await readdir(basepath);
	setupCommon(watcher, basepath, initial_filenames, cur);
}
export async function setupRecursive(watcher: IDiffWatcher, basepath: string, cntr: DocumentAccessor) {
	const initial_document = await cntr.findByPath(basepath);
	const initial_filenames = initial_document.map((x) => x.filename);
	const cur = await readdir(basepath, { withFileTypes: true });
	setupCommon(
		watcher,
		basepath,
		initial_filenames,
		cur.map((x) => x.name),
	);
	await Promise.all([cur.filter((x) => x.isDirectory()).map((x) => setupHelp(watcher, join(basepath, x.name), cntr))]);
}
