import type { Context } from "koa";
import Router from "koa-router";
import { createReadableStreamFromZip, entriesByNaturalOrder, readZip } from "../util/zipwrap.ts";
import type { ContentContext } from "./context.ts";
import { since_last_modified } from "./util.ts";
import { Readable } from "node:stream";


async function renderZipImage(ctx: Context, path: string, page: number) {
	const image_ext = ["gif", "png", "jpeg", "bmp", "webp", "jpg", "avif"];
	const zip = await readZip(path);
	const entries = (await entriesByNaturalOrder(zip.reader)).filter((x) => {
		const ext = x.filename.split(".").pop();
		return ext !== undefined && image_ext.includes(ext);
	});
	if (0 <= page && page < entries.length) {
		const entry = entries[page];
		const last_modified = entry.lastModDate;
		if (since_last_modified(ctx, last_modified)) {
			return;
		}
		const read_stream = await createReadableStreamFromZip(zip.reader, entry);
		const nodeReadableStream = new Readable();
		nodeReadableStream._read = () => { };

		read_stream.pipeTo(new WritableStream({
			write(chunk) {
				nodeReadableStream.push(chunk);
			},
			close() {
				nodeReadableStream.push(null);
			},
		}));
		nodeReadableStream.on("error", (err) => {
			console.error("readalbe stream error",err);
			ctx.status = 500;
			ctx.body = "Internal Server Error";
			zip.reader.close();
			return;
		});
		nodeReadableStream.on("close", () => {
			zip.reader.close();
		});

		ctx.body = nodeReadableStream;
		ctx.response.length = entry.uncompressedSize;
		ctx.response.type = entry.filename.split(".").pop() as string;
		ctx.status = 200;
		ctx.set("Date", new Date().toUTCString());
		ctx.set("Last-Modified", last_modified.toUTCString());
	} else {
		ctx.status = 404;
	}
}

export class ComicRouter extends Router<ContentContext> {
	constructor() {
		super();
		this.get("/", async (ctx, next) => {
			await renderZipImage(ctx, ctx.state.location.path, 0);
		});
		this.get("/:page(\\d+)", async (ctx, next) => {
			const page = Number.parseInt(ctx.params.page);
			await renderZipImage(ctx, ctx.state.location.path, page);
		});
		this.get("/thumbnail", async (ctx, next) => {
			await renderZipImage(ctx, ctx.state.location.path, 0);
		});
	}
}

export default ComicRouter;
